searchNodes=[{"doc":"An Elixir implementation for Zipper based on the paper Functional pearl: the zipper . by Gérard Huet. ZipperEx provides functions to handle an traverse tree data structures. Usage To create a zipper you can use the ZipperEx.Zipable protocol or create a module. Creating a zipper module Imagine we have a tree structure based on a tuple with an integer value and a list of cildren. The leafs of the tree are also integers. A tree may then look like the following: { 1 , [ 2 , { 3 , [ 4 , 5 ] } , 6 , { 7 , [ 0 ] } ] } # 1 # +-+-+---+-+ # 2 3 6 7 # +-+-+ + # 4 5 0 To handle this tree we create the following module: defmodule Support.Zipper do use ZipperEx @impl ZipperEx def branch? ( item ) do case item do { _ , [ _ | _ ] } -&gt; true _else -&gt; false end end @impl ZipperEx def children ( { _ , children } ) , do : children @impl ZipperEx def make_node ( { value , _children } , children ) , do : { value , children } def make_node ( value , children ) , do : { value , children } end For use ZipperEx we have to implement the callbacks branch?/1 , children/1 and make_node/2 . The Support.Zipper module contains all functions the ZipperEx also provides. We can now use Support.Zipper . iex&gt; alias Support.Zipper iex&gt; zipper = Zipper . new ( { 1 , [ 2 , { 3 , [ 4 , 5 ] } , 6 , { 7 , [ 0 ] } ] } ) # ZipperEx &lt; { 1 , [ 2 , { 3 , [ 4 , 5 ] } , 6 , { 7 , [ 0 ] } ] } &gt; iex&gt; zipper = Zipper . down ( zipper ) # ZipperEx &lt; 2 &gt; iex&gt; zipper = Zipper . right ( zipper ) # ZipperEx &lt; { 3 , [ 4 , 5 ] } &gt; iex&gt; Zipper . remove ( zipper ) |&gt; Zipper . root ( ) { 1 , [ 2 , 6 , { 7 , [ 0 ] } ] } Using the ZipperEx.Zipable protocol Similar to the module example the protocol needs implementations for ZipperEx.Zipable.branch?/1 , ZipperEx.Zipable.children/1 and ZipperEx.Zipable.make_node/2 . defmodule Support.TreeNode do @moduledoc false defstruct value : nil , children : [ ] def new ( { value , children } ) do struct! ( __MODULE__ , value : value , children : Enum . map ( children , &amp; new / 1 ) ) end def new ( value ) , do : struct! ( __MODULE__ , value : value ) defimpl ZipperEx.Zipable do def branch? ( %{ children : [ _ | _ ] } ) , do : true def branch? ( _node ) , do : false def children ( %{ children : children } ) , do : children def make_node ( node , children ) , do : %{ node | children : children } end defimpl Inspect do def inspect ( node , _opts ) , do : &quot;#TreeNode&lt; \#{ node . value } , \#{ inspect ( node . children ) } &gt;&quot; end end Support.TreeNode can then be used as follows: iex&gt; alias Support.TreeNode iex&gt; tree = TreeNode . new ( { 1 , [ 2 , { 3 , [ 4 , 5 ] } ] } ) iex&gt; zipper = ZipperEx . new ( tree ) iex&gt; ZipperEx . down ( zipper ) # ZipperEx &lt; # TreeNode &lt; 2 , [ ] &gt; &gt; iex&gt; zipper |&gt; ZipperEx . down ( ) |&gt; ZipperEx . rightmost ( ) # ZipperEx &lt; # TreeNode &lt; 3 , [ # TreeNode &lt; 4 , [ ] &gt; , # TreeNode &lt; 5 , [ ] &gt; ] &gt; &gt; iex&gt; { _zipper , acc } = ZipperEx . traverse ( zipper , [ ] , fn z , acc -&gt; ...&gt; node = ZipperEx . node ( z ) ...&gt; { z , [ node . value | acc ] } ...&gt; end ) iex&gt; acc [ 5 , 4 , 3 , 2 , 1 ] Example supporters The module Support.Zipper and Support.TreeNode are also used in the examples for this module.","ref":"ZipperEx.html","title":"ZipperEx","type":"behaviour"},{"doc":"Appends a child to the given zipper . Examples iex&gt; alias Support.Zipper iex&gt; zipper = Zipper . new ( 1 ) # ZipperEx &lt; 1 &gt; iex&gt; zipper = Zipper . append_child ( zipper , 2 ) # ZipperEx &lt; { 1 , [ 2 ] } &gt; iex&gt; Zipper . append_child ( zipper , 3 ) # ZipperEx &lt; { 1 , [ 2 , 3 ] } &gt; iex&gt; alias Support.TreeNode iex&gt; zipper = ZipperEx . new ( TreeNode . new ( { 1 , [ 2 ] } ) ) iex&gt; ZipperEx . append_child ( zipper , TreeNode . new ( 3 ) ) # ZipperEx &lt; # TreeNode &lt; 1 , [ # TreeNode &lt; 2 , [ ] &gt; , # TreeNode &lt; 3 , [ ] &gt; ] &gt; &gt;","ref":"ZipperEx.html#append_child/2","title":"ZipperEx.append_child/2","type":"function"},{"doc":"Returns true if the given node is a branch.","ref":"ZipperEx.html#branch?/1","title":"ZipperEx.branch?/1","type":"function"},{"doc":"Should return true if the given node is a branch.","ref":"ZipperEx.html#c:branch?/1","title":"ZipperEx.branch?/1","type":"callback"},{"doc":"Returns the children of the given node .","ref":"ZipperEx.html#children/1","title":"ZipperEx.children/1","type":"function"},{"doc":"Returns the children of the given node .","ref":"ZipperEx.html#c:children/1","title":"ZipperEx.children/1","type":"callback"},{"doc":"Returns the leftmost child node of the given zipper . Returns nil if the zipper is not a branch. Examples iex&gt; alias Support.TreeNode iex&gt; zipper = ZipperEx . new ( TreeNode . new ( { 1 , [ 2 , 3 ] } ) ) iex&gt; zipper = ZipperEx . down ( zipper ) # ZipperEx &lt; # TreeNode &lt; 2 , [ ] &gt; &gt; iex&gt; ZipperEx . down ( zipper ) nil","ref":"ZipperEx.html#down/1","title":"ZipperEx.down/1","type":"function"},{"doc":"Returns true when the zipper has reached the end. A zipper reached his end by using traverse/2 , traverse/3 or calling next/1 until the end. Examples iex&gt; zipper = Support.Zipper . new ( { 1 , [ 2 ] } ) iex&gt; ZipperEx . end? ( zipper ) false iex&gt; zipper |&gt; ZipperEx . traverse ( &amp; Function . identity / 1 ) |&gt; ZipperEx . end? ( ) true iex&gt; zipper |&gt; ZipperEx . next ( ) |&gt; ZipperEx . next ( ) |&gt; ZipperEx . end? ( ) true","ref":"ZipperEx.html#end?/1","title":"ZipperEx.end?/1","type":"function"},{"doc":"Returns the first zipper for which fun returns a truthy value. If no such zipper is found, returns nil. Runs through the tree in a depth-first pre-order. Examples iex&gt; alias Support.TreeNode iex&gt; zipper = ZipperEx . new ( TreeNode . new ( { 1 , [ 2 , 3 ] } ) ) iex&gt; ZipperEx . find ( zipper , fn node -&gt; ZipperEx . branch? ( node ) == false end ) # ZipperEx &lt; # TreeNode &lt; 2 , [ ] &gt; &gt;","ref":"ZipperEx.html#find/2","title":"ZipperEx.find/2","type":"function"},{"doc":"Inserts a child to the given zipper at leftmost position. Examples iex&gt; alias Support.TreeNode iex&gt; zipper = ZipperEx . new ( TreeNode . new ( { 1 , [ 3 ] } ) ) iex&gt; ZipperEx . insert_child ( zipper , TreeNode . new ( 2 ) ) # ZipperEx &lt; # TreeNode &lt; 1 , [ # TreeNode &lt; 2 , [ ] &gt; , # TreeNode &lt; 3 , [ ] &gt; ] &gt; &gt;","ref":"ZipperEx.html#insert_child/2","title":"ZipperEx.insert_child/2","type":"function"},{"doc":"Inserts a child as a left sibling to the given zipper. Raises an ArgumentError when called with an top level zipper. Examples iex&gt; alias Support.TreeNode iex&gt; zipper = ...&gt; TreeNode . new ( { 1 , [ 3 ] } ) ...&gt; |&gt; ZipperEx . new ( ) ...&gt; |&gt; ZipperEx . down ( ) iex&gt; zipper |&gt; ZipperEx . insert_left ( TreeNode . new ( 2 ) ) |&gt; ZipperEx . top ( ) # ZipperEx &lt; # TreeNode &lt; 1 , [ # TreeNode &lt; 2 , [ ] &gt; , # TreeNode &lt; 3 , [ ] &gt; ] &gt; &gt;","ref":"ZipperEx.html#insert_left/2","title":"ZipperEx.insert_left/2","type":"function"},{"doc":"Inserts a child as a right sibling to the given zipper. Raises an ArgumentError when called with an top level zipper. Examples iex&gt; alias Support.TreeNode iex&gt; zipper = ...&gt; TreeNode . new ( { 1 , [ 2 ] } ) ...&gt; |&gt; ZipperEx . new ( ) ...&gt; |&gt; ZipperEx . down ( ) iex&gt; zipper |&gt; ZipperEx . insert_right ( TreeNode . new ( 3 ) ) |&gt; ZipperEx . top ( ) # ZipperEx &lt; # TreeNode &lt; 1 , [ # TreeNode &lt; 2 , [ ] &gt; , # TreeNode &lt; 3 , [ ] &gt; ] &gt; &gt;","ref":"ZipperEx.html#insert_right/2","title":"ZipperEx.insert_right/2","type":"function"},{"doc":"Returns the left sibling of the given zipper . Returns nil if the zipper doesn't have a left sibling. Examples iex&gt; alias Support.TreeNode iex&gt; zipper = ...&gt; TreeNode . new ( { 1 , [ 10 , 11 ] } ) ...&gt; |&gt; ZipperEx . new ( ) ...&gt; |&gt; ZipperEx . down ( ) iex&gt; zipper = ZipperEx . right ( zipper ) # ZipperEx &lt; # TreeNode &lt; 11 , [ ] &gt; &gt; iex&gt; ZipperEx . right ( zipper ) nil iex&gt; zipper = ZipperEx . left ( zipper ) # ZipperEx &lt; # TreeNode &lt; 10 , [ ] &gt; &gt; iex&gt; ZipperEx . left ( zipper ) nil","ref":"ZipperEx.html#left/1","title":"ZipperEx.left/1","type":"function"},{"doc":"Returns the leftmost sibling of the given zipper . Returns the zipper himself if the given zipper is the leftmost sibling. Examples iex&gt; alias Support.TreeNode iex&gt; zipper = ...&gt; TreeNode . new ( { 1 , [ 10 , 11 , 12 ] } ) ...&gt; |&gt; ZipperEx . new ( ) ...&gt; |&gt; ZipperEx . down ( ) iex&gt; zipper = ZipperEx . rightmost ( zipper ) # ZipperEx &lt; # TreeNode &lt; 12 , [ ] &gt; &gt; iex&gt; ZipperEx . leftmost ( zipper ) # ZipperEx &lt; # TreeNode &lt; 10 , [ ] &gt; &gt;","ref":"ZipperEx.html#leftmost/1","title":"ZipperEx.leftmost/1","type":"function"},{"doc":"Creates a node from the given node and children .","ref":"ZipperEx.html#make_node/2","title":"ZipperEx.make_node/2","type":"function"},{"doc":"Creates a node from the given node and children .","ref":"ZipperEx.html#c:make_node/2","title":"ZipperEx.make_node/2","type":"callback"},{"doc":"Returns a zipper where each node is the result of invoking fun on each corresponding node of the zipper. Runs through the tree in depth-first per-order. Examples iex&gt; alias Support.Zipper iex&gt; zipper = Zipper . new ( { 1 , [ 2 , { 3 , [ 400 , 500 ] } , 6 ] } ) iex&gt; Zipper . map ( zipper , fn ...&gt; { value , children } -&gt; { value * 2 , children } ...&gt; value -&gt; value * 2 ...&gt; end ) # ZipperEx &lt; { 2 , [ 4 , { 6 , [ 800 , 1000 ] } , 12 ] } &gt;","ref":"ZipperEx.html#map/2","title":"ZipperEx.map/2","type":"function"},{"doc":"Returns a new %ZipperEx{} from a given tree or %ZipperEx . Examples iex&gt; alias Support.Zipper iex&gt; zipper = Support.Zipper . new ( { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } ) # ZipperEx &lt; { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } &gt; iex&gt; zipper |&gt; ZipperEx . next ( ) |&gt; ZipperEx . new ( ) # ZipperEx &lt; { 2 , [ 3 , 4 ] } &gt; iex&gt; zipper |&gt; Zipper . next ( ) |&gt; ZipperEx . new ( ) # ZipperEx &lt; { 2 , [ 3 , 4 ] } &gt; iex&gt; alias Support.TreeNode iex&gt; zipper = ZipperEx . new ( TreeNode . new ( { 1 , [ 2 ] } ) ) # ZipperEx &lt; # TreeNode &lt; 1 , [ # TreeNode &lt; 2 , [ ] &gt; ] &gt; &gt; iex&gt; zipper |&gt; ZipperEx . next ( ) |&gt; ZipperEx . new ( ) # ZipperEx &lt; # TreeNode &lt; 2 , [ ] &gt; &gt;","ref":"ZipperEx.html#new/1","title":"ZipperEx.new/1","type":"function"},{"doc":"Returns the next zipper for the given zipper . The function walks through the tree in a depth-first pre-order. After the last Returns to the root after the last zipper and marks the zipper as ended. An ended zipper is detectable via end?/1 . Calling next/1 for an ended zipper returns the given zipper . Examples iex&gt; zipper = Support.Zipper . new ( { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } ) # ZipperEx &lt; { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } &gt; iex&gt; zipper = ZipperEx . next ( zipper ) # ZipperEx &lt; { 2 , [ 3 , 4 ] } &gt; iex&gt; zipper = ZipperEx . next ( zipper ) # ZipperEx &lt; 3 &gt; iex&gt; zipper = ZipperEx . next ( zipper ) # ZipperEx &lt; 4 &gt; iex&gt; zipper = ZipperEx . next ( zipper ) # ZipperEx &lt; 5 &gt; iex&gt; zipper = ZipperEx . next ( zipper ) # ZipperEx &lt; { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } &gt; iex&gt; ZipperEx . next ( zipper ) # ZipperEx &lt; { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } &gt; iex&gt; ZipperEx . end? ( zipper ) true","ref":"ZipperEx.html#next/1","title":"ZipperEx.next/1","type":"function"},{"doc":"Returns the node from the given zipper . Examples iex&gt; zipper = Support.Zipper . new ( { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } ) # ZipperEx &lt; { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } &gt; iex&gt; ZipperEx . node ( zipper ) { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] }","ref":"ZipperEx.html#node/1","title":"ZipperEx.node/1","type":"function"},{"doc":"Returns the previours zipper for the given zipper . The function walks through the tree in the opposit direction as next/1 . If no previous zipper is availble, nil will be returned. Examples iex&gt; zipper = Support.Zipper . new ( { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } ) # ZipperEx &lt; { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } &gt; iex&gt; zipper = ZipperEx . next ( zipper ) # ZipperEx &lt; { 2 , [ 3 , 4 ] } &gt; iex&gt; zipper = ZipperEx . next ( zipper ) # ZipperEx &lt; 3 &gt; iex&gt; zipper = ZipperEx . prev ( zipper ) # ZipperEx &lt; { 2 , [ 3 , 4 ] } &gt; iex&gt; zipper = ZipperEx . prev ( zipper ) # ZipperEx &lt; { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } &gt; iex&gt; ZipperEx . prev ( zipper ) nil","ref":"ZipperEx.html#prev/1","title":"ZipperEx.prev/1","type":"function"},{"doc":"Removes the given zipper . The function returns the previous zipper . If remove/1 is called with an top level zipper an ArgumentError will be raised. Examples iex&gt; zipper = Support.Zipper . new ( { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } ) # ZipperEx &lt; { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } &gt; iex&gt; zipper = ZipperEx . down ( zipper ) # ZipperEx &lt; { 2 , [ 3 , 4 ] } &gt; iex&gt; ZipperEx . remove ( zipper ) # ZipperEx &lt; { 1 , [ 5 ] } &gt;","ref":"ZipperEx.html#remove/1","title":"ZipperEx.remove/1","type":"function"},{"doc":"Replaces the zipper with a zipper with the given node . Examples iex&gt; zipper = Support.Zipper . new ( { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } ) # ZipperEx &lt; { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } &gt; iex&gt; zipper |&gt; ZipperEx . down ( ) |&gt; ZipperEx . replace ( 7 ) |&gt; ZipperEx . root ( ) { 1 , [ 7 , 5 ] }","ref":"ZipperEx.html#replace/2","title":"ZipperEx.replace/2","type":"function"},{"doc":"Returns the right sibling of the given zipper . Returns nil if the zipper doesn't have a right sibling. Examples iex&gt; alias Support.TreeNode iex&gt; zipper = ...&gt; TreeNode . new ( { 1 , [ 10 , 11 ] } ) ...&gt; |&gt; ZipperEx . new ( ) ...&gt; |&gt; ZipperEx . down ( ) iex&gt; zipper = ZipperEx . right ( zipper ) # ZipperEx &lt; # TreeNode &lt; 11 , [ ] &gt; &gt; iex&gt; ZipperEx . right ( zipper ) nil iex&gt; zipper = ZipperEx . left ( zipper ) # ZipperEx &lt; # TreeNode &lt; 10 , [ ] &gt; &gt; iex&gt; ZipperEx . left ( zipper ) nil","ref":"ZipperEx.html#right/1","title":"ZipperEx.right/1","type":"function"},{"doc":"Returns the rightmost sibling of the given zipper . Returns the zipper himself if the given zipper is the rightmost sibling. Examples iex&gt; alias Support.TreeNode iex&gt; zipper = ...&gt; TreeNode . new ( { 1 , [ 10 , 11 , 12 ] } ) ...&gt; |&gt; ZipperEx . new ( ) ...&gt; |&gt; ZipperEx . down ( ) iex&gt; zipper = ZipperEx . rightmost ( zipper ) # ZipperEx &lt; # TreeNode &lt; 12 , [ ] &gt; &gt; iex&gt; ZipperEx . rightmost ( zipper ) # ZipperEx &lt; # TreeNode &lt; 12 , [ ] &gt; &gt;","ref":"ZipperEx.html#rightmost/1","title":"ZipperEx.rightmost/1","type":"function"},{"doc":"Returns the root node. Examples iex&gt; alias Support.Zipper iex&gt; zipper = Zipper . new ( { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } ) # ZipperEx &lt; { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } &gt; iex&gt; zipper = zipper |&gt; Zipper . down ( ) |&gt; Zipper . down ( ) # ZipperEx &lt; 3 &gt; iex&gt; Zipper . root ( zipper ) { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] }","ref":"ZipperEx.html#root/1","title":"ZipperEx.root/1","type":"function"},{"doc":"Returns the top level zipper. Examples iex&gt; alias Support.Zipper iex&gt; zipper = Zipper . new ( { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } ) # ZipperEx &lt; { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } &gt; iex&gt; zipper = zipper |&gt; Zipper . down ( ) |&gt; Zipper . down ( ) # ZipperEx &lt; 3 &gt; iex&gt; Zipper . top ( zipper ) # ZipperEx &lt; { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } &gt;","ref":"ZipperEx.html#top/1","title":"ZipperEx.top/1","type":"function"},{"doc":"Traverses the tree for the given zipper in depth-first pre-order and invokes fun for each zipper along the way. Examples iex&gt; zipper = Support.Zipper . new ( { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } ) iex&gt; ZipperEx . traverse ( zipper , fn z -&gt; ...&gt; ZipperEx . update ( z , fn ...&gt; { value , children } -&gt; { value + 100 , children } ...&gt; value -&gt; value + 200 ...&gt; end ) ...&gt; end ) # ZipperEx &lt; { 101 , [ { 102 , [ 203 , 204 ] } , 205 ] } &gt;","ref":"ZipperEx.html#traverse/2","title":"ZipperEx.traverse/2","type":"function"},{"doc":"Traverses the tree for the given zipper in depth-first pre-order and invokes fun for each zipper along the way with the accumulator. Examples iex&gt; zipper = Support.Zipper . new ( { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } ) iex&gt; { zipper , acc } = ZipperEx . traverse ( zipper , [ ] , fn z , acc -&gt; ...&gt; updated = ZipperEx . update ( z , fn ...&gt; { value , children } -&gt; { value + 100 , children } ...&gt; value -&gt; value + 200 ...&gt; end ) ...&gt; { updated , [ ZipperEx . node ( z ) | acc ] } ...&gt; end ) iex&gt; zipper # ZipperEx &lt; { 101 , [ { 102 , [ 203 , 204 ] } , 205 ] } &gt; iex&gt; acc [ 5 , 4 , 3 , { 2 , [ 3 , 4 ] } , { 1 , [ { 2 , [ 3 , 4 ] } , 5 ] } ]","ref":"ZipperEx.html#traverse/3","title":"ZipperEx.traverse/3","type":"function"},{"doc":"Returns the parent zipper of the given zipper . Returns nil if the zipper is the root. Examples iex&gt; alias Support.TreeNode iex&gt; zipper = ZipperEx . new ( TreeNode . new ( { 1 , [ 2 , 3 ] } ) ) iex&gt; zipper = ZipperEx . down ( zipper ) # ZipperEx &lt; # TreeNode &lt; 2 , [ ] &gt; &gt; iex&gt; ZipperEx . up ( zipper ) # ZipperEx &lt; # TreeNode &lt; 1 , [ # TreeNode &lt; 2 , [ ] &gt; , # TreeNode &lt; 3 , [ ] &gt; ] &gt; &gt;","ref":"ZipperEx.html#up/1","title":"ZipperEx.up/1","type":"function"},{"doc":"Updates the node of the given zipper . iex&gt; alias Support.TreeNode iex&gt; zipper = ZipperEx . new ( TreeNode . new ( { 1 , [ 2 , 3 ] } ) ) iex&gt; zipper = ZipperEx . down ( zipper ) iex&gt; zipper = ZipperEx . update ( zipper , fn % TreeNode { } = node -&gt; ...&gt; %{ node | value : 99 } ...&gt; end ) # ZipperEx &lt; # TreeNode &lt; 99 , [ ] &gt; &gt; iex&gt; ZipperEx . root ( zipper ) # TreeNode &lt; 1 , [ # TreeNode &lt; 99 , [ ] &gt; , # TreeNode &lt; 3 , [ ] &gt; ] &gt;","ref":"ZipperEx.html#update/2","title":"ZipperEx.update/2","type":"function"},{"doc":"","ref":"ZipperEx.html#t:t/0","title":"ZipperEx.t/0","type":"type"},{"doc":"","ref":"ZipperEx.html#t:t/1","title":"ZipperEx.t/1","type":"opaque"},{"doc":"","ref":"ZipperEx.html#t:tree/0","title":"ZipperEx.tree/0","type":"type"},{"doc":"A zipper implementation for nested lists. Examples iex&gt; zipper = ZipperEx.List . new ( [ 1 , [ 2 , [ 3 , 4 ] , 5 ] , 6 ] ) iex&gt; { _ , acc } = ZipperEx . traverse ( zipper , [ ] , fn z , acc -&gt; ...&gt; { z , [ ZipperEx . node ( z ) | acc ] } ...&gt; end ) ...&gt; Enum . reverse ( acc ) [ [ 1 , [ 2 , [ 3 , 4 ] , 5 ] , 6 ] , 1 , [ 2 , [ 3 , 4 ] , 5 ] , 2 , [ 3 , 4 ] , 3 , 4 , 5 , 6 ]","ref":"ZipperEx.List.html","title":"ZipperEx.List","type":"module"},{"doc":"See ZipperEx.append_child/2 .","ref":"ZipperEx.List.html#append_child/2","title":"ZipperEx.List.append_child/2","type":"function"},{"doc":"See ZipperEx.down/1 .","ref":"ZipperEx.List.html#down/1","title":"ZipperEx.List.down/1","type":"function"},{"doc":"See ZipperEx.end?/1 .","ref":"ZipperEx.List.html#end?/1","title":"ZipperEx.List.end?/1","type":"function"},{"doc":"See ZipperEx.find/2 .","ref":"ZipperEx.List.html#find/2","title":"ZipperEx.List.find/2","type":"function"},{"doc":"See ZipperEx.insert_child/2 .","ref":"ZipperEx.List.html#insert_child/2","title":"ZipperEx.List.insert_child/2","type":"function"},{"doc":"See ZipperEx.insert_left/2 .","ref":"ZipperEx.List.html#insert_left/2","title":"ZipperEx.List.insert_left/2","type":"function"},{"doc":"See ZipperEx.insert_right/2 .","ref":"ZipperEx.List.html#insert_right/2","title":"ZipperEx.List.insert_right/2","type":"function"},{"doc":"See ZipperEx.left/1 .","ref":"ZipperEx.List.html#left/1","title":"ZipperEx.List.left/1","type":"function"},{"doc":"See ZipperEx.leftmost/1 .","ref":"ZipperEx.List.html#leftmost/1","title":"ZipperEx.List.leftmost/1","type":"function"},{"doc":"See ZipperEx.map/2 .","ref":"ZipperEx.List.html#map/2","title":"ZipperEx.List.map/2","type":"function"},{"doc":"","ref":"ZipperEx.List.html#new/1","title":"ZipperEx.List.new/1","type":"function"},{"doc":"See ZipperEx.next/1 .","ref":"ZipperEx.List.html#next/1","title":"ZipperEx.List.next/1","type":"function"},{"doc":"See ZipperEx.node/1 .","ref":"ZipperEx.List.html#node/1","title":"ZipperEx.List.node/1","type":"function"},{"doc":"See ZipperEx.prev/1 .","ref":"ZipperEx.List.html#prev/1","title":"ZipperEx.List.prev/1","type":"function"},{"doc":"See ZipperEx.remove/1 .","ref":"ZipperEx.List.html#remove/1","title":"ZipperEx.List.remove/1","type":"function"},{"doc":"See ZipperEx.replace/2 .","ref":"ZipperEx.List.html#replace/2","title":"ZipperEx.List.replace/2","type":"function"},{"doc":"See ZipperEx.right/1 .","ref":"ZipperEx.List.html#right/1","title":"ZipperEx.List.right/1","type":"function"},{"doc":"See ZipperEx.rightmost/1 .","ref":"ZipperEx.List.html#rightmost/1","title":"ZipperEx.List.rightmost/1","type":"function"},{"doc":"See ZipperEx.root/1 .","ref":"ZipperEx.List.html#root/1","title":"ZipperEx.List.root/1","type":"function"},{"doc":"See ZipperEx.top/1 .","ref":"ZipperEx.List.html#top/1","title":"ZipperEx.List.top/1","type":"function"},{"doc":"See ZipperEx.traverse/2 .","ref":"ZipperEx.List.html#traverse/2","title":"ZipperEx.List.traverse/2","type":"function"},{"doc":"See ZipperEx.traverse/3 .","ref":"ZipperEx.List.html#traverse/3","title":"ZipperEx.List.traverse/3","type":"function"},{"doc":"See ZipperEx.up/1 .","ref":"ZipperEx.List.html#up/1","title":"ZipperEx.List.up/1","type":"function"},{"doc":"A zipper implementation for nested maps. Examples iex&gt; zipper = ZipperEx.Map . new ( %{ a : 1 , b : %{ c : 2 , d : 3 } } ) iex&gt; { _ , acc } = ZipperEx . traverse ( zipper , [ ] , fn z , acc -&gt; ...&gt; { z , [ ZipperEx . node ( z ) | acc ] } ...&gt; end ) ...&gt; Enum . reverse ( acc ) [ %{ a : 1 , b : %{ c : 2 , d : 3 } } , { :a , 1 } , { :b , %{ c : 2 , d : 3 } } , { :c , 2 } , { :d , 3 } ]","ref":"ZipperEx.Map.html","title":"ZipperEx.Map","type":"module"},{"doc":"See ZipperEx.append_child/2 .","ref":"ZipperEx.Map.html#append_child/2","title":"ZipperEx.Map.append_child/2","type":"function"},{"doc":"See ZipperEx.down/1 .","ref":"ZipperEx.Map.html#down/1","title":"ZipperEx.Map.down/1","type":"function"},{"doc":"See ZipperEx.end?/1 .","ref":"ZipperEx.Map.html#end?/1","title":"ZipperEx.Map.end?/1","type":"function"},{"doc":"See ZipperEx.find/2 .","ref":"ZipperEx.Map.html#find/2","title":"ZipperEx.Map.find/2","type":"function"},{"doc":"See ZipperEx.insert_child/2 .","ref":"ZipperEx.Map.html#insert_child/2","title":"ZipperEx.Map.insert_child/2","type":"function"},{"doc":"See ZipperEx.insert_left/2 .","ref":"ZipperEx.Map.html#insert_left/2","title":"ZipperEx.Map.insert_left/2","type":"function"},{"doc":"See ZipperEx.insert_right/2 .","ref":"ZipperEx.Map.html#insert_right/2","title":"ZipperEx.Map.insert_right/2","type":"function"},{"doc":"See ZipperEx.left/1 .","ref":"ZipperEx.Map.html#left/1","title":"ZipperEx.Map.left/1","type":"function"},{"doc":"See ZipperEx.leftmost/1 .","ref":"ZipperEx.Map.html#leftmost/1","title":"ZipperEx.Map.leftmost/1","type":"function"},{"doc":"See ZipperEx.map/2 .","ref":"ZipperEx.Map.html#map/2","title":"ZipperEx.Map.map/2","type":"function"},{"doc":"","ref":"ZipperEx.Map.html#new/1","title":"ZipperEx.Map.new/1","type":"function"},{"doc":"See ZipperEx.next/1 .","ref":"ZipperEx.Map.html#next/1","title":"ZipperEx.Map.next/1","type":"function"},{"doc":"See ZipperEx.node/1 .","ref":"ZipperEx.Map.html#node/1","title":"ZipperEx.Map.node/1","type":"function"},{"doc":"See ZipperEx.prev/1 .","ref":"ZipperEx.Map.html#prev/1","title":"ZipperEx.Map.prev/1","type":"function"},{"doc":"See ZipperEx.remove/1 .","ref":"ZipperEx.Map.html#remove/1","title":"ZipperEx.Map.remove/1","type":"function"},{"doc":"See ZipperEx.replace/2 .","ref":"ZipperEx.Map.html#replace/2","title":"ZipperEx.Map.replace/2","type":"function"},{"doc":"See ZipperEx.right/1 .","ref":"ZipperEx.Map.html#right/1","title":"ZipperEx.Map.right/1","type":"function"},{"doc":"See ZipperEx.rightmost/1 .","ref":"ZipperEx.Map.html#rightmost/1","title":"ZipperEx.Map.rightmost/1","type":"function"},{"doc":"See ZipperEx.root/1 .","ref":"ZipperEx.Map.html#root/1","title":"ZipperEx.Map.root/1","type":"function"},{"doc":"See ZipperEx.top/1 .","ref":"ZipperEx.Map.html#top/1","title":"ZipperEx.Map.top/1","type":"function"},{"doc":"See ZipperEx.traverse/2 .","ref":"ZipperEx.Map.html#traverse/2","title":"ZipperEx.Map.traverse/2","type":"function"},{"doc":"See ZipperEx.traverse/3 .","ref":"ZipperEx.Map.html#traverse/3","title":"ZipperEx.Map.traverse/3","type":"function"},{"doc":"See ZipperEx.up/1 .","ref":"ZipperEx.Map.html#up/1","title":"ZipperEx.Map.up/1","type":"function"},{"doc":"Zipable protocol used by ZipperEx . See ZipperEx for an example.","ref":"ZipperEx.Zipable.html","title":"ZipperEx.Zipable","type":"protocol"},{"doc":"Returns true if the given node is a branch.","ref":"ZipperEx.Zipable.html#branch?/1","title":"ZipperEx.Zipable.branch?/1","type":"function"},{"doc":"Returns the children of the given node .","ref":"ZipperEx.Zipable.html#children/1","title":"ZipperEx.Zipable.children/1","type":"function"},{"doc":"Creates a node from the given node and children .","ref":"ZipperEx.Zipable.html#make_node/2","title":"ZipperEx.Zipable.make_node/2","type":"function"},{"doc":"","ref":"ZipperEx.Zipable.html#t:t/0","title":"ZipperEx.Zipable.t/0","type":"type"}]